/*
 * This file is part of Adblock Plus <https://adblockplus.org/>,
 * Copyright (C) 2006-2015 Eyeo GmbH
 *
 * Adblock Plus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * Adblock Plus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
 */

(function(global)
{
  if (!global.ext)
    global.ext = require("ext_background");

  var Prefs = require("prefs").Prefs;
  var Utils = require("utils").Utils;
  var FilterStorage = require("filterStorage").FilterStorage;
  var FilterNotifier = require("filterNotifier").FilterNotifier;
  var defaultMatcher = require("matcher").defaultMatcher;
  
  var filterClasses = require("filterClasses");
  var Filter = filterClasses.Filter;
  var BlockingFilter = filterClasses.BlockingFilter;
  var Synchronizer = require("synchronizer").Synchronizer;

  var subscriptionClasses = require("subscriptionClasses");
  var Subscription = subscriptionClasses.Subscription;
  var DownloadableSubscription = subscriptionClasses.DownloadableSubscription;
  var SpecialSubscription = subscriptionClasses.SpecialSubscription;

  function convertObject(keys, obj)
  {
    var result = {};
    for (var i = 0; i < keys.length; i++)
      result[keys[i]] = obj[keys[i]];
    return result;
  }

  var convertSubscription = convertObject.bind(null, ["disabled",
    "downloadStatus", "homepage", "lastSuccess", "title", "url"]);
  var convertFilter = convertObject.bind(null, ["text"]);

  var changeListeners = null;
  var messageTypes = {
    "app": "app.listen",
    "filter": "filters.listen",
    "subscription": "subscriptions.listen"
  };

  function sendMessage(type, action, args, page)
  {
    var pages = page ? [page] : changeListeners.keys();
    for (var i = 0; i < pages.length; i++)
    {
      var filters = changeListeners.get(pages[i]);
      if (filters[type] && filters[type].indexOf(action) >= 0)
      {
        pages[i].sendMessage({
          type:  messageTypes[type],
          action: action,
          args: args
        });
      }
    }
  }

  function onFilterChange(action)
  {
    if (action == "load")
      action = "filter.loaded";

    var parts = action.split(".", 2);
    var type;
    if (parts.length == 1)
    {
      type = "app";
      action = parts[0];
    }
    else
    {
      type = parts[0];
      action = parts[1];
    }

    if (!messageTypes.hasOwnProperty(type))
      return;

    var args = Array.prototype.slice.call(arguments, 1).map(function(arg)
    {
      if (arg instanceof Subscription)
        return convertSubscription(arg);
      else if (arg instanceof Filter)
        return convertFilter(arg);
      else
        return arg;
    });
    sendMessage(type, action, args);
  }

  global.ext.onMessage.addListener(function(message, sender, callback)
  {
    var listenerFilters = null;
    switch (message.type)
    {
      case "app.listen":
      case "filters.listen":
      case "subscriptions.listen":
        if (!changeListeners)
        {
          changeListeners = new global.ext.PageMap();
          FilterNotifier.addListener(onFilterChange);
        }

        listenerFilters = changeListeners.get(sender.page);
        if (!listenerFilters)
        {
          listenerFilters = Object.create(null);
          changeListeners.set(sender.page, listenerFilters);
        }
        break;
    }

    switch (message.type)
    {
      case "add-subscription":
        ext.showOptions(function()
        {
          var subscription = Subscription.fromURL(message.url);
          subscription.title = message.title;
          onFilterChange("addSubscription", subscription);
        });
        break;
      case "app.get":
        if (message.what == "issues")
        {
          var info = require("info");
          callback({
            seenDataCorruption: "seenDataCorruption" in global ? global.seenDataCorruption : false,
            filterlistsReinitialized: "filterlistsReinitialized" in global ? global.filterlistsReinitialized : false,
            legacySafariVersion: (info.platform == "safari" && (
                Services.vc.compare(info.platformVersion, "6.0") < 0 ||   // beforeload breaks websites in Safari 5
                Services.vc.compare(info.platformVersion, "6.1") == 0 ||  // extensions are broken in 6.1 and 7.0
                Services.vc.compare(info.platformVersion, "7.0") == 0))
          });
        }
        else if (message.what == "doclink")
          callback(Utils.getDocLink(message.link));
        else if (message.what == "localeInfo")
        {
          var bidiDir;
          if ("chromeRegistry" in Utils)
            bidiDir = Utils.chromeRegistry.isLocaleRTL("adblockplus") ? "rtl" : "ltr";
          else
            bidiDir = ext.i18n.getMessage("@@bidi_dir");

          callback({locale: Utils.appLocale, bidiDir: bidiDir});
        }
        else if (message.what == "addonVersion")
        {
          callback(require("info").addonVersion);
        }
        else
          callback(null);
        break;
      case "app.listen":
        if (message.filter)
          listenerFilters.app = message.filter;
        else
          delete listenerFilters.app;
        break;
      case "app.open":
        if (message.what == "options")
          ext.showOptions();
        break;
      case "filters.add":
        var filter = Filter.fromText(message.text);
        var result = require("filterValidation").parseFilter(message.text);
        if (result.error)
          sendMessage("app", "error", [result.error.toString()], sender.page);
        else if (result.filter)
          FilterStorage.addFilter(result.filter);
        break;
      case "filters.blocked":
        var filter = defaultMatcher.matchesAny(message.url, message.requestType,
          message.docDomain, message.thirdParty);
        callback(filter instanceof BlockingFilter);
        break;
      case "filters.get":
        var subscription = Subscription.fromURL(message.subscriptionUrl);
        if (!subscription)
        {
          callback([]);
          break;
        }
        
        callback(subscription.filters.map(convertFilter));
        break;
      case "filters.importRaw":
        var result = require("filterValidation").parseFilters(message.text);
        var errors = [];
        for (var i = 0; i < result.errors.length; i++)
        {
          var error = result.errors[i];
          if (error.type != "unexpected-filter-list-header")
            errors.push(error.toString());
        }

        if (errors.length > 0)
        {
          sendMessage("app", "error", errors, sender.page);
          return;
        }

        var seenFilter = Object.create(null);
        for (var i = 0; i < result.filters.length; i++)
        {
          var filter = result.filters[i];
          FilterStorage.addFilter(filter);
          seenFilter[filter.text] = null;
        }

        for (var i = 0; i < FilterStorage.subscriptions.length; i++)
        {
          var subscription = FilterStorage.subscriptions[i];
          if (!(subscription instanceof SpecialSubscription))
            continue;

          for (var j = subscription.filters.length - 1; j >= 0; j--)
          {
            var filter = subscription.filters[j];
            if (/^@@\|\|([^\/:]+)\^\$document$/.test(filter.text))
              continue;

            if (!(filter.text in seenFilter))
              FilterStorage.removeFilter(filter);
          }
        }
        break;
      case "filters.listen":
        if (message.filter)
          listenerFilters.filter = message.filter;
        else
          delete listenerFilters.filter;
        break;
      case "filters.remove":
        var filter = Filter.fromText(message.text);
        var subscription = null;
        if (message.subscriptionUrl)
          subscription = Subscription.fromURL(message.subscriptionUrl);

        if (!subscription)
          FilterStorage.removeFilter(filter);
        else
          FilterStorage.removeFilter(filter, subscription, message.index);
        break;
      case "prefs.get":
        callback(Prefs[message.key]);
        break;
      case "subscriptions.add":
        if (message.url in FilterStorage.knownSubscriptions)
          return;

        var subscription = Subscription.fromURL(message.url);
        if (!subscription)
          return;

        subscription.disabled = false;
        if ("title" in message)
          subscription.title = message.title;
        if ("homepage" in message)
          subscription.homepage = message.homepage;
        FilterStorage.addSubscription(subscription);

        if (subscription instanceof DownloadableSubscription && !subscription.lastDownload)
          Synchronizer.execute(subscription);
        break;
      case "subscriptions.get":
        var subscriptions = FilterStorage.subscriptions.filter(function(s)
        {
          if (message.ignoreDisabled && s.disabled)
            return false;
          if (s instanceof DownloadableSubscription && message.downloadable)
            return true;
          if (s instanceof SpecialSubscription && message.special)
            return true;
          return false;
        });
        callback(subscriptions.map(convertSubscription));
        break;
      case "subscriptions.listen":
        if (message.filter)
          listenerFilters.subscription = message.filter;
        else
          delete listenerFilters.subscription;
        break;
      case "subscriptions.remove":
        var subscription = Subscription.fromURL(message.url);
        if (subscription.url in FilterStorage.knownSubscriptions)
          FilterStorage.removeSubscription(subscription);
        break;
      case "subscriptions.toggle":
        var subscription = Subscription.fromURL(message.url);
        if (subscription.url in FilterStorage.knownSubscriptions && !subscription.disabled)
          FilterStorage.removeSubscription(subscription);
        else
        {
          subscription.disabled = false;
          subscription.title = message.title;
          subscription.homepage = message.homepage;
          FilterStorage.addSubscription(subscription);
          if (!subscription.lastDownload)
            Synchronizer.execute(subscription);
        }
        break;
    }
  });
})(this);
